---------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------
library IEEE;
	use IEEE.std_logic_1164.all;
	use IEEE.std_logic_UNSIGNED.all;
library UNISIM;
	use UNISIM.vcomponents.all;
---------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------
entity oversample_4x is
    generic (
        C_NUM_CHANNELS    : integer range 1 to 50 := 2
    );
	port (
		Clk0          : in std_logic;
        Clk90         : in std_logic;
        Rst          : in std_logic;

		din           : in std_logic_vector(C_NUM_CHANNELS-1 downto 0);
		din_bufout    : out std_logic_vector(C_NUM_CHANNELS-1 downto 0);
	    dout          : out std_logic_vector(C_NUM_CHANNELS*4-1 downto 0)
	);
end oversample_4x;
---------------------------------------------------------------------------------------------
-- Architecture section
---------------------------------------------------------------------------------------------
architecture oversample_4x_struct of oversample_4x is
---------------------------------------------------------------------------------------------
-- Component Instantiation
---------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------
-- Constants, Signals and Attributes Declarations
---------------------------------------------------------------------------------------------

-- Constants
constant Low  : std_logic	:= '0';
constant High : std_logic	:= '1';
-- Signals
signal IntClk0_n    : std_logic;
signal IntClk90_n   : std_logic;

signal IntClk0    : std_logic;
signal IntClk90   : std_logic;

-- Attributes

---------------------------------------------------------------------------------------------
begin

-- we need BUFIO!

--   BUFIO_inst : BUFIO
--   port map (
--      O => IntClk0, -- 1-bit output: Clock output (connect to I/O clock loads).
--      I => Clk0  -- 1-bit input: Clock input (connect to an IBUF or BUFMR).
--   );

--   BUFIO_inst90 : BUFIO
--   port map (
--      O => IntClk90, -- 1-bit output: Clock output (connect to I/O clock loads).
--      I => Clk90  -- 1-bit input: Clock input (connect to an IBUF or BUFMR).
--   );

IntClk0 <= Clk0;
IntClk90 <= Clk90;


IntClk0_n <= not IntClk0;
IntClk90_n <= not IntClk90;
--
--
--
--
ISERDES_Gen:
   for i in 0 to C_NUM_CHANNELS-1 generate
      begin
   
ISERDES_Inst : ISERDESE2
    generic map (
        INTERFACE_TYPE      => "OVERSAMPLE",
        DATA_RATE           => "DDR", 
        DATA_WIDTH          => 4, 
        OFB_USED            => "FALSE",
        NUM_CE              => 1,
        SERDES_MODE         => "MASTER",
        IOBDELAY            => "NONE",
        DYN_CLKDIV_INV_EN   => "FALSE",
        DYN_CLK_INV_EN      => "FALSE",
        INIT_Q1             => '0',
        INIT_Q2             => '0',
        INIT_Q3             => '0',
        INIT_Q4             => '0',
        SRVAL_Q1            => '0',
        SRVAL_Q2            => '0',
        SRVAL_Q3            => '0',
        SRVAL_Q4            => '0'
    )
    port map (
        CLK             => IntClk0,
        CLKB            => IntClk0_n,
        OCLK            => IntClk90,
        OCLKB           => IntClk90_n,
        D               => din(i),
        BITSLIP         => Low,
        CE1             => High,
        CE2             => High,
        CLKDIV          => Low,
        CLKDIVP         => Low,
        DDLY            => Low,
        DYNCLKDIVSEL    => Low,
        DYNCLKSEL       => Low,
        OFB             => Low,
        RST             => Rst,
        SHIFTIN1        => Low,
        SHIFTIN2        => Low,
        O               => din_bufout(i),
        Q1              => dout(i*4+0),
        Q2              => dout(i*4+2),
        Q3              => dout(i*4+1),
        Q4              => dout(i*4+3),
        Q5              => open,
        Q6              => open,
        Q7              => open,
        Q8              => open,
        SHIFTOUT1       => open,
        SHIFTOUT2       => open
    );

      end generate;  

---------------------------------------------------------------------------------------------
end oversample_4x_struct;
--